/**
 * apeman app to handle sign.
 * @function apemanAppSign
 * @param {Object} UserSign - UserSignModel.
 * @param {Object} [options] - Optional settings.
 * @param {Object} [options.signCreateSchema] - Schema for sign create.
 * @param {Object} [options.signUpdateSchema] - Schema for sign update.
 * @param {Object} [options.signPassCreateSchema] - Schema for sign pass create.
 * @param {number} [options.saltLength=24] - Length of salt.
 * @param {number} [options.passwordSpec='pdkdf2:120:84'] - Password algorithm, iterations, length.
 * @returns {function} - Defined app function.
 */

'use strict'

const rest = require('apeman-app-rest')
const assert = require('assert')
const co = require('co')
const defineDigest = require('./digesting/define_digest')

const schemas = require('./schemas')

const {
  SignCreateEndpoint,
  SignDestroyEndpoint,
  SignPassCreateEndpoint,
  SignPassDestroyEndpoint,
  SignPassOneEndpoint,
  SignUpdateEndpoint
} = require('./endpoints')

/** @lends create */
function create (UserSign, options = {}) {
  assert.ok(UserSign, 'UserSign is required.')
  assert.ok(!!(UserSign.$$inherited && UserSign.$$inherited.ApUserSign), 'UserSign should inherit ApUserSign.')

  let { pathname } = options
  pathname = pathname || '/sign'

  return [
    co.wrap(function * setup (ctx, next) {
      ctx.state.digest = defineDigest(options.passwordSpec)
      yield next()
    }),
    ...rest(UserSign, {
      pathname: `${pathname}/pass`,

      create: new SignPassCreateEndpoint({}),
      update: false,
      bulkUpdate: false,
      destroy: new SignPassDestroyEndpoint({}),
      bulkDestroy: false,
      one: new SignPassOneEndpoint({}),
      list: false,
      relatedList: false,
      relatedCreate: false,
      relatedUpdateBulk: false,
      relatedDestroyBulk: false,
      relatedOne: false,
      relatedUpdate: false,
      relatedDestroy: false,
      relationList: false,
      relationCreate: false,
      relationUpdate: false,
      relationDestroy: false
    }),
    ...rest(UserSign, {
      pathname,
      create: new SignCreateEndpoint({}),
      update: new SignUpdateEndpoint({}),
      bulkUpdate: false,
      destroy: new SignDestroyEndpoint({}),
      bulkDestroy: false,
      relatedList: false,
      relatedCreate: false,
      relatedUpdateBulk: false,
      relatedDestroyBulk: false,
      relatedOne: false,
      relatedUpdate: false,
      relatedDestroy: false,
      relationList: false,
      relationCreate: false,
      relationUpdate: false,
      relationDestroy: false
    })
  ]
}

module.exports = create
