/**
 * Endpoint to create sign.
 * @class SignCreateEndpoint
 */
'use strict'

const { CreateEndpoint } = require('apeman-app-rest/lib/endpoints')
const schemas = require('../schemas')
const { toType, toData, defineCreate } = require('apemanmodel')
const uuid = require('uuid')
const co = require('co')

/** @lends SignCreateEndpoint */
class SignCreateEndpoint extends CreateEndpoint {
  /**
   * @override
   */
  getDefaultConfig () {
    return {
      name: 'signCreate',
      schema: schemas('sign-create')
    }
  }

  /**
   * @override
   */
  handle (ctx, next) {
    const s = this
    let succeed = s.bindSucceed(ctx)
    let fail = s.bindFail(ctx)

    return co(function * create () {
      let { state } = ctx
      let { data, model } = state
      let error = s.validate(state.data, state)
      if (error) {
        return fail(error)
      }

      let created = yield s.create(data, state)

      return succeed(
        { status: 201 },
        s.format(toData(model, created, {})),
        { self: `./${created.id}` }
      )
    })
  }

  /**
   * @override
   */
  create (data, state) {
    const s = this
    let create = super.create.bind(s)
    let { digest } = state
    return co(function * () {
      let { attributes } = data

      attributes.hashSalt = uuid.v4()
      attributes.hash = yield digest(attributes.password, attributes.hashSalt)
      attributes.signupAt = new Date()

      delete attributes.password

      data.attributes = attributes
      return yield create(data, state)
    })
  }

  format (data) {
    let { attributes } = data

    delete attributes.hash
    delete attributes.hashSalt
    return Object.assign(data, { attributes })
  }
}

module.exports = SignCreateEndpoint
