/**
 * Endpoint for sign pass destroy
 * @class SignPassDestroyEndpoint
 */
'use strict'

const { DestroyEndpoint } = require('apeman-app-rest/lib/endpoints')

const errors = require('../errors')
const co = require('co')
const { toType } = require('apemanmodel')
const { ApSignPassModel } = require('../models')

/** @lends SignPassDestroyEndpoint */
class SignPassDestroyEndpoint extends DestroyEndpoint {
  /**
   * @override
   */
  getDefaultConfig () {
    return {
      name: 'signPassDestroy'
    }
  }

  handle (ctx, next) {
    const s = this
    let succeed = s.bindSucceed(ctx)
    let fail = s.bindFail(ctx)
    let type = toType(ApSignPassModel)

    return co(function * () {
      let { state } = ctx
      let { id } = state
      let signed = ctx.signed()
      if (!signed) {
        return succeed({ count: 0 })
      }
      if (signed.id !== id) {
        return fail(errors.notFoundError(type, id))
      }
      ctx.signed({})
      return succeed({ count: 1 })
    })
  }

}

module.exports = SignPassDestroyEndpoint
