/**
 * Endpoint for sign pass one
 * @class SignPassOneEndpoint
 */
'use strict'

const { OneEndpoint } = require('apeman-app-rest/lib/endpoints')
const errors = require('../errors')
const co = require('co')
const { toData, toType } = require('apemanmodel')
const { ApSignPassModel } = require('../models')

/** @lends SignPassOneEndpoint */
class SignPassOneEndpoint extends OneEndpoint {
  /**
   * @override
   */
  getDefaultConfig () {
    return {
      name: 'signPassOne',
      singleton: true
    }
  }

  /**
   * @override
   */
  handle (ctx, next) {
    const s = this
    let succeed = s.bindSucceed(ctx)
    let fail = s.bindFail(ctx)
    let type = toType(ApSignPassModel)

    return co(function * create () {
      let { state } = ctx
      let { id, model } = state
      let { pass, sign } = ctx.signed()
      if (!pass) {
        return fail(errors.notFoundError(type, id))
      }
      return succeed(
        {},
        pass,
        {},
        [
          s.format(toData(model, sign))
        ]
      )
    })
  }

  format (data) {
    let { attributes } = data

    delete attributes.hash
    delete attributes.hashSalt
    return Object.assign(data, { attributes })
  }
}

module.exports = SignPassOneEndpoint
