/**
 * Endpoint for sign update.
 * @class SignUpdateEndpoint
 */
'use strict'

const { UpdateEndpoint } = require('apeman-app-rest/lib/endpoints')
const schemas = require('../schemas')
const co = require('co')

/** @lends SignUpdateEndpoint */
class SignUpdateEndpoint extends UpdateEndpoint {
  /**
   * @override
   */
  getDefaultConfig () {
    return {
      name: 'signUpdate',
      schema: schemas('sign-update')
    }
  }

  /**
   * @override
   */
  update (instance, data, state) {
    const s = this
    let update = super.update.bind(s)
    let { digest } = state
    return co(function * () {
      let { attributes } = data

      // Ignore given hash and salt
      delete attributes.hash
      delete attributes.hashSalt

      attributes.signmodAt = new Date()

      if (attributes.password) {
        attributes.hash = yield digest(attributes.password, instance.hashSalt)
        delete attributes.password
      }

      data.attributes = attributes
      data = yield update(instance, data, state)
      attributes = data.attributes

      delete attributes.hash
      delete attributes.hashSalt
      Object.assign(data, {
        attributes
      })

      return data
    })
  }
}

module.exports = SignUpdateEndpoint
